import { EC2 } from 'aws-sdk';
/***********************************
 * Types for the Extension contract
 ***********************************/
export interface Extension {
    loadHandelExtension(context: ExtensionContext): void | Promise<void>;
}
export interface ExtensionContext {
    service(name: string, deployer: ServiceDeployer): this;
}
/***********************************
 * Types for the Service Deployer contract
 ***********************************/
export interface ServiceDeployer {
    providedEventType: ServiceEventType | null;
    producedEventsSupportedTypes: ServiceEventType[];
    producedDeployOutputTypes: DeployOutputType[];
    consumedDeployOutputTypes: DeployOutputType[];
    supportsTagging: boolean;
    /**
     * Checks the given service configuration in the user's Handel file for required parameters and correctness.
     * This provides a fail-fast mechanism for configuration errors before deploy is attempted.
     *
     * You should probably always implement this phase in every service deployer
     */
    check?(serviceContext: ServiceContext<ServiceConfig>, dependenciesServiceContexts: Array<ServiceContext<ServiceConfig>>): string[];
    /**
     * Create resources needed for deployment that are also needed for dependency wiring
     * with other services.
     *
     * Implement this phase if you'll be creating security groups for any of your resources
     *
     * Example AWS services that woulod need to implement this phase include Beanstalk and RDS
     *
     * NOTE: If you implement preDeploy, you must implement getPreDeployContext as well
     */
    preDeploy?(serviceContext: ServiceContext<ServiceConfig>): Promise<PreDeployContext>;
    /**
     * Get the PreDeploy context information without running preDeploy
     *
     * Return null if preDeploy has not been executed yet
     */
    getPreDeployContext?(serviceContext: ServiceContext<ServiceConfig>): Promise<IPreDeployContext>;
    /**
     * Bind two resources from the preDeploy phase together by performing some wiring action on them. An example
     * is to add an ingress rule from one security group onto another.
     *
     * Bind is run from the perspective of the service being consumed, not the other way around. In other words, it
     * is run on the dependency who is adding the ingress rule for the dependent service.
     *
     * Implement this phase if you'll be creating resources that need to add ingress rules for dependent services
     * to talk to them
     *
     * Example AWS services that would need to implement this phase include RDS and EFS
     */
    bind?(ownServiceContext: ServiceContext<ServiceConfig>, ownPreDeployContext: IPreDeployContext, dependentOfServiceContext: ServiceContext<ServiceConfig>, dependentOfPreDeployContext: IPreDeployContext): Promise<IBindContext>;
    /**
     * Deploy the resources contained in your service deployer.
     *
     * You are responsible for using the outputs in the dependenciesDeployContexts to wire up this service
     * to those. For example, each one may return an IAM policiy that you should add to whatever role is
     * created for your service.
     *
     * All this service's dependencies are guaranteed to be deployed before this phase gets called
     */
    deploy?(ownServiceContext: ServiceContext<ServiceConfig>, ownPreDeployContext: IPreDeployContext, dependenciesDeployContexts: IDeployContext[]): Promise<IDeployContext>;
    /**
     * In this phase, this service should make any changes necessary to allow it to consume events from the given source
     * For example, a Lambda consuming events from an SNS topic should add a Lambda Function Permission to itself to allow
     * the SNS ARN to invoke it.
     *
     * This method will only be called if your service is listed as an event consumer in another service's configuration.
     */
    consumeEvents?(ownServiceContext: ServiceContext<ServiceConfig>, ownDeployContext: IDeployContext, eventConsumerConfig: ServiceEventConsumer, producerServiceContext: ServiceContext<ServiceConfig>, producerDeployContext: IDeployContext): Promise<IConsumeEventsContext>;
    /**
     * In this phase, this service should make any changes necessary to allow it to produce events to the consumer service.
     * For example, an S3 bucket producing events to a Lambda should add the event notifications to the S3 bucket for the
     * Lambda.
     *
     * This method will only be called if your service has an event_consumers element in its configruation.
     */
    produceEvents?(ownServiceContext: ServiceContext<ServiceConfig>, ownDeployContext: IDeployContext, eventConsumerConfig: ServiceEventConsumer, consumerServiceContext: ServiceContext<ServiceConfig>, consumerDeployContext: IDeployContext): Promise<IProduceEventsContext>;
    /**
     * In this phase, the service should remove all resources created in the preDeploy phase.
     *
     * Implment this phase if you implemented the preDeploy phase!
     */
    unPreDeploy?(ownServiceContext: ServiceContext<ServiceConfig>): Promise<IUnPreDeployContext>;
    /**
     * In this phase, the service should remove all bindings on preDeploy resources.
     */
    unBind?(ownServiceContext: ServiceContext<ServiceConfig>, ownPreDeployContext: IPreDeployContext, dependentOfServiceContext: ServiceContext<ServiceConfig>, dependentOfPreDeployContext: IPreDeployContext): Promise<IUnBindContext>;
    /**
     * In this phase, the service should delete resources created during the deploy phase.
     *
     * Note that there are no 'unConsumeEvents' or 'unProduceEvents' phases. In most cases, deleting the
     * service will automatically delete any event bindings the service itself has, but in some cases this phase will
     * also need to manually remove event bindings. An example of this is CloudWatch Events, which requires that
     * you remove all targets before you can delete the service.
     */
    unDeploy?(ownServiceContext: ServiceContext<ServiceConfig>): Promise<IUnDeployContext>;
}
export declare enum DeployOutputType {
    SecurityGroups = "SecurityGroups",
    Policies = "Policies",
    EnvironmentVariables = "EnvironmentVariables",
    Scripts = "Scripts",
    Credentials = "Credentials"
}
export declare enum ServiceEventType {
    SNS = "SNS",
    Lambda = "Lambda",
    SQS = "SQS",
    CloudWatchEvents = "CloudWatchEvents",
    S3 = "S3",
    DynamoDB = "DynamoDB",
    IoT = "IoT",
    AlexaSkillKit = "AlexaSkillKit"
}
/***********************************
 * Types for the Account Config File
 ***********************************/
export interface AccountConfig {
    account_id: string;
    region: string;
    vpc: string;
    public_subnets: string[];
    private_subnets: string[];
    data_subnets: string[];
    ssh_bastion_sg?: string;
    elasticache_subnet_group: string;
    rds_subnet_group: string;
    redshift_subnet_group: string;
    required_tags?: string[];
    handel_resource_tags?: Tags;
    [key: string]: any;
}
export declare function isAccountConfig(obj: any): obj is AccountConfig;
/***********************************
 * Types for the context objects used by service deployers
 ***********************************/
export interface ServiceRegistry {
    getService(prefix: string, name: string): ServiceDeployer;
    getService(type: ServiceType): ServiceDeployer;
    hasService(prefix: string, name: string): boolean;
    hasService(type: ServiceType): boolean;
    allPrefixes(): Set<string>;
}
export interface IServiceType {
    prefix: string;
    name: string;
    matches(prefix: string, name: string): boolean;
}
export declare function isServiceType(obj: any | ServiceType): obj is ServiceType;
export interface IServiceContext<Config extends ServiceConfig> extends HasAppServiceInfo {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    params: Config;
    accountConfig: AccountConfig;
    serviceInfo: ServiceInfo;
    tags: Tags;
    resourceName(): string;
    stackName(): string;
    injectedEnvVars(outputs: any): any;
    ssmApplicationPrefix(): string;
    ssmParamName(suffix: string): string;
}
export declare function isServiceContext(obj: any): obj is ServiceContext<ServiceConfig>;
export interface ServiceInfo {
    producedEventsSupportedTypes: ServiceEventType[];
    producedDeployOutputTypes: string[];
    consumedDeployOutputTypes: string[];
}
export declare function isServiceInfo(obj: any): obj is ServiceInfo;
export interface ServiceConfig {
    type: string;
    tags?: Tags;
    event_consumers?: ServiceEventConsumer[];
    dependencies?: string[];
}
export declare function isServiceConfig(obj: any): obj is ServiceConfig;
export interface ServiceEventConsumer {
    service_name: string;
}
export interface IBindContext {
    dependencyServiceContext: ServiceContext<ServiceConfig>;
    dependentOfServiceContext: ServiceContext<ServiceConfig>;
}
export declare function isBindContext(obj: any | IBindContext): obj is IBindContext;
export interface IConsumeEventsContext {
    consumingServiceContext: ServiceContext<ServiceConfig>;
    producingServiceContext: ServiceContext<ServiceConfig>;
}
export declare function isConsumeEventsContext(obj: any | IConsumeEventsContext): obj is IConsumeEventsContext;
export interface IDeployContext extends HasAppServiceInfo {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    eventOutputs: DeployContextEventOutputs | null;
    policies: any[];
    environmentVariables: DeployContextEnvironmentVariables;
    scripts: string[];
    addEnvironmentVariables(envVars: EnvironmentVariables): void;
}
export declare function isDeployContext(obj: any | IDeployContext): obj is IDeployContext;
export interface DeployContextEnvironmentVariables {
    [key: string]: string;
}
export declare function isDeployContextEnvironmentVariables(obj: any | DeployContextEnvironmentVariables): obj is DeployContextEnvironmentVariables;
export interface DeployContextEventOutputs {
    resourceName?: string;
    resourceArn?: string;
    resourcePrincipal: string;
    serviceEventType: ServiceEventType;
}
export declare function isDeployContextEventOutputs(obj: any | DeployContextEventOutputs): obj is DeployContextEventOutputs;
export interface IPreDeployContext extends HasAppServiceInfo {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    securityGroups: EC2.SecurityGroup[];
}
export declare function isPreDeployContext(obj: any | IPreDeployContext): obj is PreDeployContext;
export interface IProduceEventsContext {
    producingServiceContext: ServiceContext<ServiceConfig>;
    consumingServiceContext: ServiceContext<ServiceConfig>;
}
export declare function isProduceEventsContext(obj: any | IProduceEventsContext): obj is IProduceEventsContext;
export interface IUnDeployContext extends HasAppServiceInfo {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
}
export declare function isUnDeployContext(obj: any | IUnDeployContext): obj is IUnDeployContext;
export interface IUnBindContext {
    dependencyServiceContext: ServiceContext<ServiceConfig>;
    dependentOfServiceContext: ServiceContext<ServiceConfig>;
}
export declare function isUnBindContext(obj: any | IUnBindContext): obj is IUnBindContext;
export interface IUnPreDeployContext {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
}
export declare function isUnPreDeployContext(obj: any | IUnBindContext): obj is IUnPreDeployContext;
export interface HasAppServiceInfo {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
}
export declare function hasAppServiceInfo(obj: any | HasAppServiceInfo): obj is HasAppServiceInfo;
export declare class ServiceContext<Config extends ServiceConfig> implements IServiceContext<Config> {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    params: Config;
    accountConfig: AccountConfig;
    tags: Tags;
    serviceInfo: ServiceInfo;
    constructor(appName: string, environmentName: string, serviceName: string, serviceType: ServiceType, params: Config, accountConfig: AccountConfig, tags?: Tags, serviceInfo?: ServiceInfo);
    resourceName(): string;
    stackName(): string;
    injectedEnvVars(): EnvironmentVariables;
    ssmParamName(suffix: string): string;
    ssmApplicationPrefix(): string;
    ssmApplicationPath(): string;
}
export declare class ServiceType implements IServiceType {
    prefix: string;
    name: string;
    constructor(prefix: string, name: string);
    matches(prefix: string, name: string): boolean;
    toString(): string;
}
export declare class BindContext implements IBindContext {
    dependencyServiceContext: ServiceContext<ServiceConfig>;
    dependentOfServiceContext: ServiceContext<ServiceConfig>;
    constructor(dependencyServiceContext: ServiceContext<ServiceConfig>, dependentOfServiceContext: ServiceContext<ServiceConfig>);
}
export declare class ConsumeEventsContext implements IConsumeEventsContext {
    consumingServiceContext: ServiceContext<ServiceConfig>;
    producingServiceContext: ServiceContext<ServiceConfig>;
    constructor(consumingServiceContext: ServiceContext<ServiceConfig>, producingServiceContext: ServiceContext<ServiceConfig>);
}
export declare class DeployContext implements IDeployContext {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    eventOutputs: DeployContextEventOutputs | null;
    policies: any[];
    environmentVariables: DeployContextEnvironmentVariables;
    scripts: string[];
    constructor(serviceContext: ServiceContext<ServiceConfig>);
    addEnvironmentVariables(envVars: EnvironmentVariables): void;
    getInjectedEnvVarName(suffix: string): string;
}
export declare class PreDeployContext implements IPreDeployContext {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    securityGroups: EC2.SecurityGroup[];
    constructor(serviceContext: ServiceContext<ServiceConfig>);
}
export declare class ProduceEventsContext implements IProduceEventsContext {
    producingServiceContext: ServiceContext<ServiceConfig>;
    consumingServiceContext: ServiceContext<ServiceConfig>;
    constructor(producingServiceContext: ServiceContext<ServiceConfig>, consumingServiceContext: ServiceContext<ServiceConfig>);
}
export declare class UnDeployContext implements IUnDeployContext {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    constructor(serviceContext: ServiceContext<ServiceConfig>);
}
export declare class UnBindContext implements IUnBindContext {
    dependencyServiceContext: ServiceContext<ServiceConfig>;
    dependentOfServiceContext: ServiceContext<ServiceConfig>;
    constructor(dependencyServiceContext: ServiceContext<ServiceConfig>, dependentOfServiceContext: ServiceContext<ServiceConfig>);
}
export declare class UnPreDeployContext implements IUnPreDeployContext {
    appName: string;
    environmentName: string;
    serviceName: string;
    serviceType: ServiceType;
    constructor(serviceContext: ServiceContext<ServiceConfig>);
}
/***********************************
 * Other Types
 ***********************************/
export interface Tags {
    [key: string]: string;
}
export declare function isTags(obj: any): obj is Tags;
export interface EnvironmentVariables {
    [key: string]: string;
}
export declare function isEnvironmentVariables(obj: any): obj is EnvironmentVariables;
export declare function isHash(obj: any): boolean;
