"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DeployOutputType;
(function (DeployOutputType) {
    DeployOutputType["SecurityGroups"] = "SecurityGroups";
    DeployOutputType["Policies"] = "Policies";
    DeployOutputType["EnvironmentVariables"] = "EnvironmentVariables";
    DeployOutputType["Scripts"] = "Scripts";
    DeployOutputType["Credentials"] = "Credentials";
})(DeployOutputType = exports.DeployOutputType || (exports.DeployOutputType = {}));
var ServiceEventType;
(function (ServiceEventType) {
    ServiceEventType["SNS"] = "SNS";
    ServiceEventType["Lambda"] = "Lambda";
    ServiceEventType["SQS"] = "SQS";
    ServiceEventType["CloudWatchEvents"] = "CloudWatchEvents";
    ServiceEventType["S3"] = "S3";
    ServiceEventType["DynamoDB"] = "DynamoDB";
    ServiceEventType["IoT"] = "IoT";
    ServiceEventType["AlexaSkillKit"] = "AlexaSkillKit";
})(ServiceEventType = exports.ServiceEventType || (exports.ServiceEventType = {}));
function isAccountConfig(obj) {
    return !!obj
        && (isString(obj.account_id) || isNumber(obj.account_id)) // We allow string or number here because Handel used to treat account_id as a number, but now treats it as a string
        && isString(obj.region)
        && isString(obj.vpc);
    // TODO: We could expand this more, but I'm not sure it's a good idea, since there may be a lot of variation between account config files.
}
exports.isAccountConfig = isAccountConfig;
function isServiceType(obj) {
    return !!obj
        && isString(obj.prefix)
        && isString(obj.name);
}
exports.isServiceType = isServiceType;
function isServiceContext(obj) {
    return !!obj
        && isServiceConfig(obj.params)
        && isAccountConfig(obj.accountConfig)
        && isTags(obj.tags)
        && hasAppServiceInfo(obj);
}
exports.isServiceContext = isServiceContext;
function isServiceInfo(obj) {
    return !!obj
        && isArray(obj.producedEventsSupportedTypes, isString)
        && isArray(obj.producedDeployOutputTypes, isString)
        && isArray(obj.consumedDeployOutputTypes, isString);
}
exports.isServiceInfo = isServiceInfo;
function isServiceConfig(obj) {
    return !!obj
        && isString(obj.type)
        && (!obj.tags || isTags(obj.tags))
        && (!obj.dependencies || isArray(obj.dependencies, isString));
}
exports.isServiceConfig = isServiceConfig;
function isBindContext(obj) {
    return !!obj
        && isServiceContext(obj.dependencyServiceContext)
        && isServiceContext(obj.dependentOfServiceContext);
}
exports.isBindContext = isBindContext;
function isConsumeEventsContext(obj) {
    return !!obj
        && isServiceContext(obj.consumingServiceContext)
        && isServiceContext(obj.producingServiceContext);
}
exports.isConsumeEventsContext = isConsumeEventsContext;
function isDeployContext(obj) {
    return !!obj
        && isDeployContextEventOutputs(obj.eventOutputs)
        && isArray(obj.policies)
        && isDeployContextEnvironmentVariables(obj.environmentVariables)
        && isArray(obj.scripts, isString)
        && hasAppServiceInfo(obj);
}
exports.isDeployContext = isDeployContext;
function isDeployContextEnvironmentVariables(obj) {
    return isHash(obj);
}
exports.isDeployContextEnvironmentVariables = isDeployContextEnvironmentVariables;
function isDeployContextEventOutputs(obj) {
    return isHash(obj) || obj == null;
}
exports.isDeployContextEventOutputs = isDeployContextEventOutputs;
function isPreDeployContext(obj) {
    return !!obj
        && isArray(obj.securityGroups)
        && hasAppServiceInfo(obj);
}
exports.isPreDeployContext = isPreDeployContext;
function isProduceEventsContext(obj) {
    return !!obj
        && isServiceContext(obj.producingServiceContext)
        && isServiceContext(obj.consumingServiceContext);
}
exports.isProduceEventsContext = isProduceEventsContext;
function isUnDeployContext(obj) {
    return !!obj && hasAppServiceInfo(obj);
}
exports.isUnDeployContext = isUnDeployContext;
function isUnBindContext(obj) {
    return !!obj
        && isServiceContext(obj.dependencyServiceContext)
        && isServiceContext(obj.dependentOfServiceContext);
}
exports.isUnBindContext = isUnBindContext;
function isUnPreDeployContext(obj) {
    return !!obj && hasAppServiceInfo(obj);
}
exports.isUnPreDeployContext = isUnPreDeployContext;
function hasAppServiceInfo(obj) {
    return !!obj
        && isString(obj.appName)
        && isString(obj.environmentName)
        && isString(obj.serviceName)
        && isServiceType(obj.serviceType);
}
exports.hasAppServiceInfo = hasAppServiceInfo;
class ServiceContext {
    constructor(appName, environmentName, serviceName, serviceType, params, accountConfig, tags = {}, serviceInfo = {
        consumedDeployOutputTypes: [],
        producedDeployOutputTypes: [],
        producedEventsSupportedTypes: []
    }) {
        this.appName = appName;
        this.environmentName = environmentName;
        this.serviceName = serviceName;
        this.serviceType = serviceType;
        this.params = params;
        this.accountConfig = accountConfig;
        this.tags = tags;
        this.serviceInfo = serviceInfo;
    }
    resourceName() {
        return `${this.appName}-${this.environmentName}-${this.serviceName}`;
    }
    stackName() {
        return `${this.resourceName()}-${this.serviceType.name}`;
    }
    injectedEnvVars() {
        const envVars = {};
        envVars.HANDEL_APP_NAME = this.appName;
        envVars.HANDEL_ENVIRONMENT_NAME = this.environmentName;
        envVars.HANDEL_SERVICE_NAME = this.serviceName;
        // HANDEL_PARAMETER_STORE_PREFIX is legacy method for parameter store and is being deprecated
        envVars.HANDEL_PARAMETER_STORE_PREFIX = this.ssmApplicationPrefix();
        // HANDEL_PARAMETER_STORE_PATH is the new recommended method for organizing parameters
        envVars.HANDEL_PARAMETER_STORE_PATH = this.ssmApplicationPath();
        envVars.HANDEL_REGION_NAME = this.accountConfig.region;
        return envVars;
    }
    ssmParamName(suffix) {
        const prefix = `${this.ssmApplicationPrefix()}.${this.serviceName}`;
        return `${prefix}.${suffix}`;
    }
    ssmApplicationPrefix() {
        return `${this.appName}.${this.environmentName}`;
    }
    ssmApplicationPath() {
        return `/${this.appName}/${this.environmentName}/`;
    }
}
exports.ServiceContext = ServiceContext;
class ServiceType {
    constructor(prefix, name) {
        this.prefix = prefix;
        this.name = name;
    }
    matches(prefix, name) {
        return this.prefix === prefix && this.name === name;
    }
    toString() {
        return this.prefix + '::' + this.name;
    }
}
exports.ServiceType = ServiceType;
class BindContext {
    constructor(dependencyServiceContext, dependentOfServiceContext) {
        this.dependencyServiceContext = dependencyServiceContext;
        this.dependentOfServiceContext = dependentOfServiceContext;
        // Should anything else go here?
    }
}
exports.BindContext = BindContext;
class ConsumeEventsContext {
    constructor(consumingServiceContext, producingServiceContext) {
        this.consumingServiceContext = consumingServiceContext;
        this.producingServiceContext = producingServiceContext;
        // TODO - Does anything else go here?
    }
}
exports.ConsumeEventsContext = ConsumeEventsContext;
class DeployContext {
    constructor(serviceContext) {
        this.appName = serviceContext.appName;
        this.environmentName = serviceContext.environmentName;
        this.serviceName = serviceContext.serviceName;
        this.serviceType = serviceContext.serviceType;
        this.eventOutputs = null;
        this.policies = [];
        this.environmentVariables = {};
        this.scripts = [];
    }
    addEnvironmentVariables(envVars) {
        const formattedVars = Object.keys(envVars)
            .reduce((obj, name) => {
            const envName = this.getInjectedEnvVarName(name);
            obj[envName] = envVars[name];
            return obj;
        }, {});
        Object.assign(this.environmentVariables, formattedVars);
    }
    getInjectedEnvVarName(suffix) {
        return `${this.serviceName}_${suffix}`.toUpperCase().replace(/-/g, '_');
    }
}
exports.DeployContext = DeployContext;
class PreDeployContext {
    constructor(serviceContext) {
        this.appName = serviceContext.appName;
        this.environmentName = serviceContext.environmentName;
        this.serviceName = serviceContext.serviceName;
        this.serviceType = serviceContext.serviceType;
        this.securityGroups = []; // Empty until service deployer fills it
    }
}
exports.PreDeployContext = PreDeployContext;
class ProduceEventsContext {
    constructor(producingServiceContext, consumingServiceContext) {
        this.producingServiceContext = producingServiceContext;
        this.consumingServiceContext = consumingServiceContext;
        // Does anything else go here?
    }
}
exports.ProduceEventsContext = ProduceEventsContext;
class UnDeployContext {
    constructor(serviceContext) {
        this.appName = serviceContext.appName;
        this.environmentName = serviceContext.environmentName;
        this.serviceName = serviceContext.serviceName;
        this.serviceType = serviceContext.serviceType;
    }
}
exports.UnDeployContext = UnDeployContext;
class UnBindContext {
    constructor(dependencyServiceContext, dependentOfServiceContext) {
        this.dependencyServiceContext = dependencyServiceContext;
        this.dependentOfServiceContext = dependentOfServiceContext;
        // Should anything else go here?
    }
}
exports.UnBindContext = UnBindContext;
class UnPreDeployContext {
    constructor(serviceContext) {
        this.appName = serviceContext.appName;
        this.environmentName = serviceContext.environmentName;
        this.serviceName = serviceContext.serviceName;
        this.serviceType = serviceContext.serviceType;
    }
}
exports.UnPreDeployContext = UnPreDeployContext;
function isTags(obj) {
    return isHash(obj);
}
exports.isTags = isTags;
function isEnvironmentVariables(obj) {
    return isHash(obj);
}
exports.isEnvironmentVariables = isEnvironmentVariables;
function isHash(obj) {
    return !!obj && typeof obj === 'object';
}
exports.isHash = isHash;
function isArray(value, itemType) {
    return Array.isArray(value) && (!itemType || value.every(itemType));
}
function isString(value) {
    return typeof value === 'string';
}
function isNumber(value) {
    return typeof value === 'number';
}
//# sourceMappingURL=extension-api.js.map